# -*- python -*-
# $RCSfile: labelledslider.py,v $
# $Revision: 1.10 $
# $Author: langer $
# $Date: 2005/05/13 18:43:51 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import gtk
from oof2.common import utils
from oof2.common import debug


class LabelledSlider:
    def __init__(self, value=None, vmin=0, vmax=1, step=0.01, callback=None):
        # "callback" is called when the user moves the slider.
        debug.mainthreadTest()
        self.gtk = gtk.GtkHBox()
        if value is None:
            value = vmin
        self.adjustment = gtk.GtkAdjustment(value=value,
                                            lower=vmin, upper=vmax,
                                            step_incr=step,
                                            page_incr=step)
        self.slider = gtk.GtkHScale(self.adjustment)
        self.gtk.add(self.slider)
        self.slider.set_draw_value(0)   # we'll display the value in self.entry
        self.adjustment.connect('value-changed', self.text_from_slider)

        self.recursion_suppression = 0

        self.entry = gtk.GtkEntry()
        self.gtk.add(self.entry)
        font = self.entry.get_style().font
        width = max(font.width(`vmin`), font.width(`vmax`), 9*font.width('0'))
        self.entry.set_usize(width, -1)
        self.entry.connect('changed', self.entry_changed)

        # The initial call to set_value here should *not* call the
        # callback, since if this LabelledSlider is part of a
        # ParameterWidget, the rest of the widget might not be
        # constructed yet.  So here we temporarily set the callback
        # function to None.  Then the 'value-changed' signal can be
        # used to call self.text_from_slider without also calling the
        # callback function.  (The alternative is to postpone
        # connecting the 'value-changed' signal until after calling
        # set_value, but then we'd have to explicitly set the text in
        # self.entry.  That would be uglier.)
        self.callback = None
        self.set_value(value)
        # Now it's safe to set the callback.
        self.callback = callback

        self.entry.connect('activate', self.slider_from_text)
        self.entry.connect('focus-out-event', self.entry_lost_focus)
        

    def set_digits(self, digits):
        "Sets number of digits after the decimal place."
        debug.mainthreadTest()
        self.slider.set_digits(digits)

    def set_value(self, value):
        debug.mainthreadTest()
        if value < self.adjustment.lower:
            self.adjustment.set_value(self.adjustment.lower)
        elif value > self.adjustment.upper:
            self.adjustment.set_value(self.adjustment.upper)
        else:
            self.adjustment.set_value(value)
        self.adjustment.value_changed()
    def get_value(self):
        debug.mainthreadTest()
        # Always return the value from the GtkEntry, not the slider.
        # The entry is always updated when the slider changes, but the
        # slider isn't necessarily updated when the entry changes.  If
        # the user puts a bad value in the entry, we want an error
        # message to be returned.
        return self.clip(utils.OOFeval(self.entry.get_text()))
    def text_from_slider(self, obj):
        if not self.recursion_suppression:
            self.recursion_suppression = 1
            try:
                debug.mainthreadTest()
                self.entry.set_text("%-8g" % self.adjustment.value)
                self.entry.set_position(0)
                if self.callback:
                    self.callback(self, self.adjustment.value)
            finally:
                self.recursion_suppression = 0
    def slider_from_text(self, obj):    # callback for 'activate' from GtkEntry
        if not self.recursion_suppression:
            self.recursion_suppression = 1
            try:
                debug.mainthreadTest()
                try:
                    v0 = utils.OOFeval(self.entry.get_text())
                except:
                    # Illegal values will raise an exception, but they
                    # may be incomplete entries.  So don't do anything
                    # about it here.
                    pass
                else:
                    self.adjustment.set_value(self.clip(v0))
            finally:
                self.recursion_suppression = 0
    def entry_lost_focus(self, obj, event):
        self.slider_from_text(obj)
    def entry_changed(self, obj):
        self.slider_from_text(obj)
    def clip(self, v):
        v = max(v, self.adjustment.lower)
        v = min(v, self.adjustment.upper)
        return v
