# -*- python -*-
# $RCSfile: pixelgroupparam.py,v $
# $Revision: 1.18 $
# $Author: langer $
# $Date: 2005/05/13 18:43:49 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common.IO import parameter
from oof2.common.IO import selectionplaceholder
import types, struct

# Parameter subclass for choosing a PixelGroup in a Microstructure

class PixelGroupParameter(parameter.StringParameter):
    def valueDesc(self):
        return "The name of a PixelGroup."

# Parameter subclass for choosing either a PixelGroup or the currently
# selected pixels.

class PixelAggregateParameter(parameter.ParameterBase):
    def __init__(self, name, value="", default=None, tip=None):
        self.checker = parameter.TypeChecker(
            types.StringType,
            selectionplaceholder.SelectionPlaceHolder,
            selectionplaceholder.AllPlaceHolder)
        parameter.ParameterBase.__init__(self, name, value=value,
                                         default=default, tip=tip)
    def valueDesc(self):
        return "The name of a PixelGroup, or <link linkend='Object:Placeholder'><constant>all</constant></link> or <link linkend='Object:Placeholder'><constant>selected</constant></link>."

    def binaryRepr(self, datafile, value):
        if isinstance(value, selectionplaceholder.PlaceHolder):
            return struct.pack(parameter.structIntFmt, value.idno)
        else:
            # just like a StringParameter's binaryRepr, but with a
            # preceding 0 to indictate that it's not a PlaceHolder
            # object.
            return struct.pack('>ii', 0, len(value)) + value
    def binaryRead(self, parser):
        b = parser.getBytes(parameter.structIntSize)
        (idno,) = struct.unpack(parameter.structIntFmt, b)
        if idno > 0:
            return selectionplaceholder.getPlaceHolderFromID(idno)
        b = parser.getBytes(parameter.structIntSize)
        (length,) = struct.unpack(parameter.structIntFmt, b)
        return parser.getBytes(length)
        
