# -*- python -*-
# $RCSfile: skeletonPage.py,v $
# $Revision: 1.78 $
# $Author: reida $
# $Date: 2005/06/03 18:08:30 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter, reporter
from oof2.common.IO import whoville
from oof2.common.IO.GUI import historian
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine import skeletoncontext
from oof2.engine import skeletonelement
from oof2.engine import skeletonmodifier
from gtk import *
import types, string

bold = load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")  # gtk stuff

# Define some convenience variables.
OOF = mainmenu.OOF
skeletonmenu = mainmenu.OOF.Skeleton
SkeletonModifier = skeletonmodifier.SkeletonModifier

class SkeletonPage(oofGUI.MainPage):
    def __init__(self):
        
        self.sbcallbacks = [
            switchboard.requestCallback("made reservation", self.update_ok_button),
            switchboard.requestCallback("cancelled reservation", self.update_ok_button),
            switchboard.requestCallback("skeleton homogeneity changed", self.reshow_state)
            ]
        oofGUI.MainPage.__init__(self, name="Skeleton", ordering=120,
                                 tip='Construct and modify mesh skeletons')

        mainbox = GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.skelwidget = whowidget.WhoWidget(whoville.getClass('Skeleton'),
                                              callback=self.select_skeleton,
                                              scope=self)
        label = GtkLabel('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[0], expand=0, fill=0)
        label = GtkLabel('Skeleton=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.skelwidget.gtk[1], expand=0, fill=0)

        # Centered box of buttons
        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        bbox = GtkHBox(homogeneous=1, spacing=3)
        hbox.add(bbox, expand=1, fill=0)

        self.newbutton = GtkButton('New...')
        self.newbutton.connect('clicked', self.new_skeleton_CB)
        guitop.top().tooltip(self.newbutton, "Create a new skeleton from the current microstructure.")
        bbox.add(self.newbutton, expand=1, fill=1)
        
        self.simplebutton = GtkButton('Simple...')
        self.simplebutton.connect('clicked', self.simple_skeleton_CB)
        guitop.top().tooltip(self.simplebutton, "Create a new skeleton from the current microstructure in a naive fashion, by creating one quadrilateral or two triangular elements per pixel.  Material boundaries will be inherently jagged, which may cause errors in finite element solutions.")
        bbox.add(self.simplebutton, expand=1, fill=1)
        
        self.renamebutton = GtkButton('Rename...')
        self.renamebutton.connect('clicked', self.rename_skeleton_CB)
        guitop.top().tooltip(self.renamebutton, "Rename the current skeleton.")
        bbox.add(self.renamebutton, expand=1, fill=1)

        self.copybutton = GtkButton('Copy...')
        self.copybutton.connect('clicked', self.copy_skeleton_CB)
        guitop.top().tooltip(self.copybutton, "Copy the current skeleton.")
        bbox.add(self.copybutton, expand=1, fill=1)

        self.deletebutton = GtkButton('Delete')
        self.deletebutton.connect('clicked', self.delete_skeletonCB)
        guitop.top().tooltip(self.deletebutton, "Delete the current skeleton.")
        bbox.add(self.deletebutton, expand=1, fill=1)

        self.savebutton = GtkButton('Save...')
        self.savebutton.connect('clicked', self.save_skeletonCB)
        guitop.top().tooltip(self.savebutton, "Save the current skeleton to a file.")
        bbox.add(self.savebutton, expand=1, fill=1)
        
        mainpane = GtkHPaned()
        mainbox.add(mainpane)

        self.skelframe = GtkFrame(label="Skeleton Status")
        mainpane.pack1(self.skelframe, resize=1, shrink=0)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.skelframe.add(scroll)
        self.msdata = GtkText()
        self.msdata.set_word_wrap(TRUE)
        scroll.add_with_viewport(self.msdata)

        # End of left-side of skeleton info frame.

        # Start of right-side

        skelmodframe = GtkFrame(label="Skeleton Modification")
        skelmodbox = GtkVBox(spacing=3)
        skelmodframe.add(skelmodbox)

        self.skelmod = regclassfactory.RegisteredClassFactory(
                       skeletonmodifier.SkeletonModifier.registry,
                       title="method: ",
                       callback=self.skelmodCB,
                       expand=0, fill=0, scope=self)
        self.historian = historian.Historian(self.skelmod.set,
                                             self.sensitizeHistory,
                                             setCBkwargs={'interactive':1})

        skelmodbox.add(self.skelmod.gtk,expand=1,fill=1)

        # Buttons for "Previous", "OK", and "Next"
        hbox = GtkHBox()
        self.prevskelmodbutton = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkArrow(at=ARROW_LEFT))
        lbox.add(GtkLabel(' Prev'))
        self.prevskelmodbutton.add(lbox)        
        self.prevskelmodbutton.connect('clicked',self.prevskelmod)
        guitop.top().tooltip(self.prevskelmodbutton,
                             'Recall the previous skeleton modification operation.')
        hbox.add(self.prevskelmodbutton, expand=0, fill=0, padding=2)

        self.okbutton = GtkButton('OK')
        self.okbutton.connect('clicked', self.okskelmod)
        guitop.top().tooltip(self.okbutton,
                  'Perform the skeleton modification operation defined above.')
        hbox.add(self.okbutton, expand=1, fill=1, padding=5)

        self.nextskelmodbutton = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkLabel('Next '), expand=0, fill=0)
        lbox.add(GtkArrow(at=ARROW_RIGHT), expand=0,fill=0)
        self.nextskelmodbutton.add(lbox)
        self.nextskelmodbutton.connect('clicked',self.nextskelmod)
        guitop.top().tooltip(self.nextskelmodbutton,
                             'Recall the next skeleton modification operation.')
        hbox.add(self.nextskelmodbutton, expand=0, fill=0, padding=2)

        skelmodbox.add(hbox, expand=0, fill=0, padding=2) 

        # Buttons for "Undo", "Redo"
        hbox = GtkHBox()
        self.undobutton = GtkButton('Undo')
        self.undobutton.connect('clicked', self.undoskelmod)
        guitop.top().tooltip(self.undobutton,
                             'Undo the latest skeleton modification.')
        hbox.add(self.undobutton, expand=1, fill=0, padding=10)

        self.redobutton = GtkButton('Redo')
        self.redobutton.connect('clicked', self.redoskelmod)
        guitop.top().tooltip(self.redobutton,
                             'Redo the latest undone skeleton modification.')
        hbox.add(self.redobutton, expand=1, fill=0, padding=10)

        skelmodbox.add(hbox, expand=0, fill=0, padding=2)

        mainpane.pack2(skelmodframe, resize=1, shrink=0)
        # End of right-side

        self.sensitize()
        self.sensitizeHistory()

        switchboard.requestCallback("Skeleton modified", self.recordModifier)
        switchboard.requestCallback(('who changed', 'Skeleton'),
                                    self.changeSkeleton)
        switchboard.requestCallback(SkeletonModifier, self.updateskelmod)
        switchboard.requestCallback(("new who", 'Image'), self.newImageMS)
        switchboard.requestCallback(("new who", 'Microstructure'),
                                    self.newImageMS)
        switchboard.requestCallback(("new who", 'Skeleton'), self.newSkeleton)
        # Pages should catch the signal from updates to the widget which
        # don't originate on this page, e.g. deletions via menu command.
        switchboard.requestCallback(self.skelwidget, self.skel_update)

        switchboard.requestCallback(('validity', self.skelmod),
                                    self.validityChangeCB)

        # Node movements can change the homogeneity of elements,
        # thus changing the state.
        switchboard.requestCallback("skeleton nodes moved",
                                    self.nodesMoved)

    def skelmodCB(self, reg):
        self.historian.stateChangeCB(reg)
        self.sensitize()

    def illegalCheck(self):
        reg = self.skelmod.getRegistration()
        if hasattr(reg, "ok_illegal"):
            return 1
        return not self.currentSkeleton().illegal()

    def currentMSName(self):
        path = labeltree.makePath(self.skelwidget.get_value())
        return path[0]

    def currentSkeletonName(self):
        path = labeltree.makePath(self.skelwidget.get_value())
        return path[-1]

    def currentSkeletonFullName(self):
        return self.skelwidget.get_value()
        

    def currentSkeleton(self):
        try:
            return skeletoncontext.skeletonContexts[self.skelwidget.get_value()].getObject()
        except KeyError:
            return None

    def select_skeleton(self, *args):
        self.set_state(self.skelwidget.get_value())

    def nodesMoved(self, context):
        cname = context.path()
        if self.skelwidget.get_value() == cname:
            self.show_state(cname)
        
    def newSkeleton(self, whoname):     # switchboard ("new who", "Skeleton")
        ## debug.fmsg("after new skeleton, before set_state")
        self.set_state(whoname)


    def skel_update(self, *args, **kwargs):  # Switchboard "self.skelwidget"
        skelname = self.skelwidget.get_value()
        self.show_state(skelname)
        
    def changeSkeleton(self, skelcontext):
        try:
            path = skelcontext.path()
        except KeyError:
            return
        if self.skelwidget.get_value()==path:
            self.set_state(path)

    def newImageMS(self, whoname):  # switchboard, new Image or Microstructure
        self.sensitize()

    def recordModifier(self, path, modifier):
        if modifier:
            self.historian.record(modifier)

    def set_state(self, skelname):
        # If switching to a new SkeletonContext, then the skelwidget
        # will change state and call its callback, which will call
        # show_state(), which displays the rest of the skeleton data
        # and sensitizes the page.  If *not* switching to a new
        # context, then the widget won't call its callback, so we have
        # to call show_state() directly.
        if labeltree.makePath(skelname) == \
               labeltree.makePath(self.skelwidget.get_value()):
            ## debug.fmsg("about to show state")
            self.show_state(skelname)
            ## launch a thread here
        else:
            self.skelwidget.set_value(skelname)
            # set_value triggers the "self.skelwidget" switchboard
            # callback, which calls show_state.

    def reshow_state(self, skel_name): # Switchboard, when MS groups change.
        if self.currentSkeletonFullName()==skel_name:
            self.show_state(skel_name)

    def show_state(self, skelname):
        ## debug.fmsg("updating skeleton page")
        debug.mainthreadTest()
        skelpath = labeltree.makePath(skelname)
        self.msdata.backward_delete(self.msdata.get_length())
        if skelpath[-1]:                # path is complete
            # Build a string, and use it to set the text of the attributes
            # GtkText box, accessible via "self.msdata".
            skel = skeletoncontext.skeletonContexts[skelpath].getObject()
            ms_name = skel.MS.name()
            nNodes = len(skel.nodes)
            nElements = len(skel.elements)
            illegalcount = 0
            shapecounts = {}            # no. of each shape of element
            for name in skeletonelement.ElementShapeType.names:
                shapecounts[name] = 0

            # Shape Count
            for e in skel.elements:
                shapecounts[e.type().name] += 1

            # Homogeneity Index stuff.  The homogIndex is "None" if
            # the skeleton in question has just been pushed on the
            # stack.  If it's not "None", we assume that it's up to
            # date, since aside from skeleton modification operations
            # (which we know about), it only changes when the
            # microstructure changes, and that's handled earlier in
            # the sequence by the same signal that called this
            # routine.
            homogIndex = skel.getHomogeneityIndex()
            if homogIndex is None:
                # sets both homogIndex and illegalCount in the skeleton.
                skel.setHomogeneityIndex()
                homogIndex = skel.getHomogeneityIndex()
            illegalcount = skel.getIllegalCount()
            
            if illegalcount > 0 :
                self.msdata.insert(bold, None, None,
                                   "WARNING: %d ILLEGAL ELEMENT%s.\n" %
                                   (illegalcount, "S"*(illegalcount!=1)) )
                self.msdata.insert(bold, None, None,
                                   "Element data is unreliable.\n")
                itORthem = ["it", "them"]
                self.msdata.insert(bold, None, None,
                                   "Remove %s before proceeding.\n\n" % itORthem[illegalcount!=1])
            
            self.msdata.insert_defaults("No. of Nodes: %d\n" % nNodes)
            self.msdata.insert_defaults("No. of Elements: %d\n" % nElements)

            for name in skeletonelement.ElementShapeType.names:
                self.msdata.insert_defaults("No. of %ss: %d\n" %
                                            (name, shapecounts[name]))
                
            self.msdata.insert_defaults("Homogeneity Index: %g\n" % homogIndex)
        else:
            self.msdata.insert_defaults("")

        self.sensitize()
        self.sensitizeHistory()

    def new_skeleton_CB(self, gtkobj): # gtk callback for "New..." button
        paramset = filter(lambda x: x.name!='microstructure',
                          skeletonmenu.New.params)
        if parameterwidgets.getParameters(title='New skeleton', *paramset):
            skeletonmenu.New.callWithDefaults(
                microstructure=self.currentMSName())

    def simple_skeleton_CB(self, gtkobj): # gtk callback for "Simple..." button
        paramset = filter(lambda x: x.name!='microstructure',
                          skeletonmenu.Simple.params)
        if parameterwidgets.getParameters(title='Simple skeleton', *paramset):
            skeletonmenu.Simple.callWithDefaults(
                microstructure=self.currentMSName())        

    def copy_skeleton_CB(self, gtkobj): # gtk callback for "Copy..." button
        menuitem = skeletonmenu.Copy
        namearg = menuitem.get_arg('name')
        if parameterwidgets.getParameters(namearg, title='Copy skeleton'):
            menuitem.callWithDefaults(skeleton=self.skelwidget.get_value())

    def rename_skeleton_CB(self, gtkobj): # gtk callback for "Rename..." button
        menuitem = skeletonmenu.Rename
        namearg = menuitem.get_arg('name')
        namearg.value = labeltree.makePath(self.skelwidget.get_value())[-1]
        if parameterwidgets.getParameters(namearg, title='Rename skeleton'):
            menuitem.callWithDefaults(skeleton=self.skelwidget.get_value())

    def delete_skeletonCB(self, gtkobj): # gtk callback for Delete button.
        skelname = self.currentSkeletonName()
        if reporter.query("Delete skeleton %s?" % skelname,
                          "OK", "Cancel", default="OK")=="OK":
            menuitem = skeletonmenu.Delete
            menuitem.callWithDefaults(skeleton=self.skelwidget.get_value())

    def getSkeletonAvailability(self):
        try:
            currentSkel = self.skelwidget.get_value() # colon separated string
            return not skeletoncontext.skeletonContexts[currentSkel].query_reservation()
        except KeyError:
            return 1
        
    def sensitize(self):
        debug.mainthreadTest()
        currentSkel = self.skelwidget.get_value() # colon separated string
        path = labeltree.makePath(currentSkel) # list of strings
        skelselected = path[-1] != ""
        self.okbutton.set_sensitive(skelselected and self.skelmod.isValid()
                                    and self.illegalCheck())
        context = skelselected and skeletoncontext.skeletonContexts[currentSkel]
        self.undobutton.set_sensitive(context and context.undoable())
        self.redobutton.set_sensitive(context and context.redoable())
        
        msclass = whoville.getClass('Microstructure')
        havems = msclass is not None and msclass.nActual() > 0
        self.newbutton.set_sensitive(havems)
        self.simplebutton.set_sensitive(havems)
        self.deletebutton.set_sensitive(skelselected)
        self.renamebutton.set_sensitive(skelselected)
        self.copybutton.set_sensitive(skelselected)
        self.savebutton.set_sensitive(skelselected)

        if not self.getSkeletonAvailability():
            self.okbutton.set_sensitive(0)
            self.undobutton.set_sensitive(0)
            self.redobutton.set_sensitive(0)
            
    def sensitizeHistory(self):
        debug.mainthreadTest()
        self.nextskelmodbutton.set_sensitive(self.historian.nextSensitive())
        self.prevskelmodbutton.set_sensitive(self.historian.prevSensitive())

    def validityChangeCB(self, validity):
        self.sensitize()

    def updateskelmod(self):
        self.skelmod.update(SkeletonModifier.registry)

    def currentFullSkeletonName(self):
        return self.skelwidget.get_value()
    
    def update_ok_button(self, whocls):
        if  self.currentFullSkeletonName() == whocls.path():
            ## print whocls.path()
            self.sensitize()
            
    def okskelmod(self,gtkobj):
        path = self.skelwidget.get_value()
        modifier = self.skelmod.get_value()
        if path and modifier:
            OOF.Skeleton.Modify(skeleton=path, modifier=modifier)
        else:
            reporter.report("Can't modify!!!!")

    def prevskelmod(self,gtkobj):
        self.historian.prevCB()

    def nextskelmod(self,gtkobj):
        self.historian.nextCB()

    def undoskelmod(self,gtkobj):
        path = self.skelwidget.get_value()
        OOF.Skeleton.Undo(skeleton = path)
        self.sensitize()
        self.sensitizeHistory()

    def redoskelmod(self,gtkobj):
        path = self.skelwidget.get_value()
        OOF.Skeleton.Redo(skeleton = path)
        self.sensitize()
        self.sensitizeHistory()

    def save_skeletonCB(self, button):
        menuitem = mainmenu.OOF.File.Save.Skeleton
        skelname = self.skelwidget.get_value()
        file = fileselector.fileSelector(
            ident='SaveSkeletonFromPage',
            title='Save Skeleton "%s"' % skelname,
            params=[menuitem.get_arg('format')])
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode,
                                          skeleton=skelname)

# Create the page.
skeletonpage = SkeletonPage()

###########################################################

from oof2.common.IO.GUI import fileselector

def _save_skel(menuitem):
    if len(skeletoncontext.skeletonContexts) > 0:
        file = fileselector.fileSelector(
            ident='SaveSkeletonFromMenu',
            title='Save Skeleton',
            params=[menuitem.get_arg('skeleton'), menuitem.get_arg('format')]
            )
        if not file is None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode)
    else:
        reporter.report("No skeletons?")

mainmenu.OOF.File.Save.Skeleton.add_gui_callback(_save_skel)

