# -*- python -*-
# $RCSfile: skeletoninfoGUI.py,v $
# $Revision: 1.44 $
# $Author: langer $
# $Date: 2005/06/09 18:36:10 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.SWIG.common import pixelgroup
from oof2.common import debug
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import primitives
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import mousehandler
from oof2.common.IO.GUI import toolboxGUI
from oof2.engine.IO import skeletoninfo
from gtk import *
import string
import math

xpadding = 3
xoptions = EXPAND|FILL

class SkeletonInfoMode:
    # Base class for ElementMode, NodeMode, and SegmentMode.
            
    def __init__(self, toolbox):
        debug.mainthreadTest()
        self.toolbox = toolbox  # SkeletonToolboxGUI
        self.menu = self.toolbox.toolbox.menu  # ie, gfxtoolbox.toolbox.menu

        self.gtk = GtkFrame(self.targetname + " Information")
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.gtk.add(scroll)
        vbox = GtkVBox()
        scroll.add_with_viewport(vbox)
        self.table = GtkTable()
        vbox.add(self.table, expand=0, fill=0)

        self.sbcallbacks = [
            switchboard.requestCallback("groupset member resized",
                                        self.grpsChanged)]
        
    def destroy(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        mainthread.runBlock(self.gtk.destroy)
        self.toolbox = None  # break circular references

    def grpsChanged(self, context, groupset):
        # Switchboard callback, for when groups have changed.
        pass

    # Utility routines for constructing GUI components
    
    def labelmaster(self, column, row, labeltext):
        debug.mainthreadTest()
        label = GtkLabel(labeltext)
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)

    def entrymaster(self, column, row, editable=0):
        debug.mainthreadTest()
        entry = GtkEntry()
        entry.set_editable(editable)
        self.table.attach(entry, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return entry

    # getBlahBlah => double-click callback
    # It is equvalent to "change mode" and "select the object".
    # showBlahBlah => single-click callback
    # It's not changing mode but shows the object.
    def getElementCB(self, element):
        self.toolbox.changeModeWithObject(element, "Element")

    def showElementCB(self, element):
        if element:
            self.toolbox.toolbox.peeker.assignObject(element, "Element")
        else:
            self.toolbox.toolbox.peeker.removeObject("Element")
        switchboard.notify("redraw")

    def getSegmentCB(self, segment):
        self.toolbox.changeModeWithObject(segment, "Segment")
        
    def showSegmentCB(self, segment):
        if segment:
            self.toolbox.toolbox.peeker.assignObject(segment, "Segment")
        else:
            self.toolbox.toolbox.peeker.removeObject("Segment")
        switchboard.notify("redraw")
        
    def getNodeCB(self, node):
        self.toolbox.changeModeWithObject(node, "Node")

    def showNodeCB(self, node):
        if node:
            self.toolbox.toolbox.peeker.assignObject(node, "Node")
        else:
            self.toolbox.toolbox.peeker.removeObject("Node")
        switchboard.notify("redraw")
        
    def menumasterElem(self, column, row, objlist=[]):
        debug.mainthreadTest()
        namelist = [None]*len(objlist)
        for i in range(len(objlist)):
            obj = objlist[i]
            name = "Element %d" % obj.index
            namelist[i] = name
        menu = chooser.FramedChooserListWidget(
            objlist=objlist, displaylist=namelist, callback=self.showElementCB,
            dbcallback=self.getElementCB, autoselect=None)
        self.table.attach(menu.gtk, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return menu

    def menumasterSgmt(self, column, row, objlist=[]):
        debug.mainthreadTest()
        namelist = [None]*len(objlist)
        for i in range(len(objlist)):
            obj = objlist[i]
            name = "Segment %d, nodes (%d, %d) (length: %g)" % \
                   (obj.index,
                    obj.nodes()[0].index,
                    obj.nodes()[1].index,
                    obj.length())
            namelist[i] = name
        menu = chooser.FramedChooserListWidget(
            objlist=objlist, displaylist=namelist, callback=self.showSegmentCB,
            dbcallback=self.getSegmentCB, autoselect=None)
        self.table.attach(menu.gtk, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return menu

    def menumasterNode(self, column, row, objlist=[]):
        debug.mainthreadTest()
        namelist = [None]*len(objlist)
        for i in range(len(objlist)):
            obj = objlist[i]
            name = "Node %d at (%g, %g)" % \
                   (obj.index, obj.position().x, obj.position().y)
            namelist[i] = name
        menu = chooser.FramedChooserListWidget(
            objlist=objlist, displaylist=namelist, callback=self.showNodeCB,
            dbcallback=self.getNodeCB, autoselect=None)
        self.table.attach(menu.gtk, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return menu

    def menumasterNodeAngle(self, column, row, element=None, objlist=[]):
        debug.mainthreadTest()
        namelist = [None]*len(objlist)
        for i in range(len(objlist)):
            obj = objlist[i]
            angle = element.getRealAngle(element.nodes.index(obj))
            name = "Node %d at (%g, %g) (angle: %g)" % \
                   (obj.index, obj.position().x, obj.position().y, angle)
            namelist[i] = name
        menu = chooser.FramedChooserListWidget(
            objlist=objlist, displaylist=namelist, callback=self.showNodeCB,
            dbcallback=self.getNodeCB, autoselect=None)
        self.table.attach(menu.gtk, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return menu    
    
    def updateGroup(self, obj):
        self.group.set_text(string.join(obj.groups, ', '))

######################
    
class ElementMode(SkeletonInfoMode):
    nrows=10
    ncols=2
    targetname = "Element"
    def __init__(self, toolbox, querier=None):
        SkeletonInfoMode.__init__(self, toolbox)

        self.labelmaster((0,1), (0,1), 'index=')
        self.index = self.entrymaster((1,2), (0,1))
        
        self.labelmaster((0,1), (1,2), 'type=')
        self.type = self.entrymaster((1,2), (1,2))        

        self.labelmaster((0,1), (2,3), 'nodes=')
        self.nodes = self.menumasterNode((1,2), (2,3))

        self.labelmaster((0,1), (3,4), 'segments=')
        self.segs = self.menumasterSgmt((1,2), (3,4))

        self.labelmaster((0,1), (4,5), 'area=')
        self.area = self.entrymaster((1,2), (4,5))

        self.labelmaster((0,1), (5,6), 'dominant pixel=')
        self.domin = self.entrymaster((1,2), (5,6), editable=1)

        self.labelmaster((0,1), (6,7), 'homogeneity=')
        self.homog = self.entrymaster((1,2), (6,7))

        self.labelmaster((0,1), (7,8), 'shape energy=')
        self.shape = self.entrymaster((1,2), (7,8))

        self.labelmaster((0,1), (8,9), 'group=')
        self.group = self.entrymaster((1,2), (8,9))

        self.labelmaster((0,1), (9,10), 'material=')
        self.material = self.entrymaster((1,2), (9,10))
        
    def menucmd(self):
        return self.menu.QueryElement

    def grpsChanged(self, context, groupset):
        querier = self.toolbox.toolbox.querier
        peeker = self.toolbox.toolbox.peeker
        if querier and querier.object:
            if groupset is querier.context.elementgroups:
                self.updateSomething(querier)
            
    def updateSomething(self, container):
        debug.mainthreadTest()
        element = container.object
        skeleton = container.skeleton
        
        self.type.set_text(`element.type()`[1:-1]) # strip quotes
        self.index.set_text(`element.getIndex()`)

        nlist = element.nodes
        self.nodes.destroy()
        self.nodes = self.menumasterNodeAngle((1,2), (2,3), element, nlist)

        self.segs.destroy()
        self.segs = self.menumasterSgmt((1,2), (3,4),
                                        element.getSegments(skeleton))
        
        self.area.set_text("%f" % element.area())

        if not element.illegal():
            domCat = element.dominantPixel(skeleton)
            repPix = skeleton.MS.getRepresentativePixel(domCat)
            pixGrp = pixelgroup.pixelGroupNames(skeleton.MS, repPix)
            name = string.join(pixGrp, ", ")
            self.domin.set_text(name)
            self.homog.set_text("%f" % element.homogeneity(skeleton))
            self.shape.set_text("%f" % element.energyShape())

            mat = element.material(skeleton)
            if mat:
                self.material.set_text(mat.name())
            else:
                self.material.set_text("<No material>")
        else:                           # illegal element
            self.domin.set_text('???')
            self.homog.set_text('???')
            self.shape.set_text('???')
            self.material.set_text('???')
        self.updateGroup(element)

    def updateNothing(self):
        debug.mainthreadTest()
        self.type.set_text("")
        self.index.set_text("")
        
        self.nodes.destroy()
        self.nodes = self.menumasterNodeAngle((1,2), (2,3))

        self.segs.destroy()
        self.segs = self.menumasterSgmt((1,2), (3,4))

        self.area.set_text("")
        self.domin.set_text("")
        self.homog.set_text("")
        self.shape.set_text("")
        self.group.set_text("")
        self.material.set_text("")

######################
    
class NodeMode(SkeletonInfoMode):
    targetname = "Node"
    nrows = 6
    ncols = 2
    def __init__(self, toolbox, querier=None):
        SkeletonInfoMode.__init__(self, toolbox)

        self.labelmaster((0,1), (0,1), 'index=')
        self.index = self.entrymaster((1,2), (0,1))

        self.labelmaster((0,1), (1,2), 'position=')
        self.pos = self.entrymaster((1,2), (1,2))

        self.labelmaster((0,1), (2,3), 'mobility=')
        self.mobility = self.entrymaster((1,2), (2,3))

        self.labelmaster((0,1), (3,4), 'elements=')
        self.elem = self.menumasterElem((1,2), (3,4))

        self.labelmaster((0,1), (4,5), 'group=')
        self.group = self.entrymaster((1,2), (4,5))

        self.labelmaster((0,1), (5,6), 'boundary=')
        self.bndy = self.entrymaster((1,2), (5,6))
        
    def grpsChanged(self, context, groupset):
        querier = self.toolbox.toolbox.querier
        peeker = self.toolbox.toolbox.peeker
        if querier and querier.object:
            if groupset is querier.context.nodegroups:
                self.updateSomething(querier)

    def updateSomething(self, container):
        debug.mainthreadTest()
        node = container.object
        skeleton = container.skeleton

        self.index.set_text(`node.getIndex()`)
        self.pos.set_text("(%g, %g)" %
                          (node.position().x, node.position().y))

        if node.movable_x() and node.movable_y():
            self.mobility.set_text("free")
        elif node.movable_x() and not node.movable_y():
            self.mobility.set_text("x only")
        elif not node.movable_x() and node.movable_y():
            self.mobility.set_text("y only")
        elif node.pinned():
            self.mobility.set_text("pinned")
        else:
            self.mobility.set_text("fixed")

        elist = node.neighborElements()
        self.elem.destroy()
        self.elem = self.menumasterElem((1,2), (3,4), elist)

        self.updateGroup(node)

        text = []
        for key in skeleton.pointboundaries.keys():
            bdy = skeleton.pointboundaries[key]
            if node in bdy.nodes:
                text.append(key)
        if text:
            bound = ""
            for txt in text:
                if bound:
                    bound += " ,"+txt
                else:
                    bound += txt
            self.bndy.set_text(bound)
        else:
            self.bndy.set_text("")

    def updateNothing(self):
        debug.mainthreadTest()
        self.index.set_text("")
        self.pos.set_text("")
        self.mobility.set_text("")
        
        self.elem.destroy()
        self.elem = self.menumasterElem((1,2), (3,4))

        self.group.set_text("")
        self.bndy.set_text("")

    def menucmd(self):
        return self.menu.QueryNode

######################
    
class SegmentMode(SkeletonInfoMode):
    targetname = "Segment"
    nrows = 6
    ncols = 2
    def __init__(self, toolbox, querier=None):
        SkeletonInfoMode.__init__(self, toolbox)
        self.labelmaster((0,1), (0,1), 'index=')
        self.index = self.entrymaster((1,2), (0,1))

        self.labelmaster((0,1), (1,2), 'nodes=')
        self.nodes = self.menumasterNode((1,2), (1,2))

        self.labelmaster((0,1), (2,3), 'elements=')
        self.elem = self.menumasterElem((1,2), (2,3))

        self.labelmaster((0,1), (3,4), 'length=')
        self.length = self.entrymaster((1,2), (3,4))

        self.labelmaster((0,1), (4,5), 'group=')
        self.group = self.entrymaster((1,2), (4,5))

        self.labelmaster((0,1), (5,6), 'boundary=')
        self.bndy = self.entrymaster((1,2), (5,6))

    def grpsChanged(self, context, groupset):
        querier = self.toolbox.toolbox.querier
        peeker = self.toolbox.toolbox.peeker
        if querier and querier.object:
            if groupset is querier.context.segmentgroups:
                self.updateSomething(querier)
            
    def updateSomething(self, container):
        debug.mainthreadTest()
        segment = container.object
        skeleton = container.skeleton

        self.index.set_text(`segment.getIndex()`)

        nlist = segment.get_nodes()
        self.nodes.destroy()
        self.nodes = self.menumasterNode((1,2), (1,2), nlist)

        elist = segment.getElements()
        self.elem.destroy()
        self.elem = self.menumasterElem((1,2), (2,3), elist)

        self.length.set_text(`segment.length()`)

        self.updateGroup(segment)

        text = []
        for key in skeleton.edgeboundaries.keys():
            bdy = skeleton.edgeboundaries[key]
            for edge in bdy.edges:
                if segment==edge.segment:
                    text.append(key)
        if text:
            bound = ""
            for txt in text:
                if bound:
                    bound += " ,"+txt
                else:
                    bound += txt
            self.bndy.set_text(bound)
        else:
            self.bndy.set_text("")
            
    def updateNothing(self):
        debug.mainthreadTest()
        self.index.set_text("")

        self.nodes.destroy()
        self.nodes = self.menumasterNode((1,2), (1,2))

        self.elem.destroy()
        self.elem = self.menumasterElem((1,2), (2,3))

        self.length.set_text("")
        self.group.set_text("")
        self.bndy.set_text("")

    def menucmd(self):
        return self.menu.QuerySegment

############################################################################

modes = [ElementMode, NodeMode, SegmentMode]

############################################################################
############################################################################

class SkeletonToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, skeletoninfo):
        toolboxGUI.GfxToolbox.__init__(self, "Skeleton Info", skeletoninfo)
        self.mainbox = GtkVBox()
        self.gtk.add(self.mainbox)

        self.modeclass = ElementMode
        self.modeobj = None
        self.modeobjdict = {}

        hbox = GtkHBox()
        self.mainbox.add(hbox, expand=0, fill=0)
        hbox.add(GtkLabel("Click on an: "), expand=0, fill=0)

        self.modebuttons = []
        self.modebuttondict = {}
        for mode in modes:
            if self.modebuttons:
                button = GtkRadioButton(label=mode.targetname,
                                        group=self.modebuttons[0])
            else:
                button = GtkRadioButton(label=mode.targetname)
            self.modebuttons.append(button)
            guitop.top().tooltip(button,
                               "Show " + mode.targetname + " Information")
            hbox.add(button, expand=0, fill=0)
            button.set_active(self.modeclass is mode)
            button.connect('clicked', self.changeModeCB, mode)
            self.modebuttondict[mode.targetname] = button
        
        self.table = GtkTable(cols=2, rows=2)
        self.mainbox.add(self.table, expand=0, fill=0)

        # Display mouse click coordinates
        label = GtkLabel('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,1, xpadding=5, xoptions=FILL)
        self.xtext = GtkEntry()
        self.xtext.set_editable(0)
        self.table.attach(self.xtext, 1,2, 0,1,
                          xpadding=5, xoptions=EXPAND|FILL)
        label = GtkLabel('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 1,2, xpadding=5, xoptions=FILL)
        self.ytext = GtkEntry()
        self.ytext.set_editable(0)
        self.table.attach(self.ytext, 1,2, 1,2,
                          xpadding=5, xoptions=EXPAND|FILL)
        self.table.attach(GtkHSeparator(), 0,2, 2,3,
                         ypadding=2, xoptions=EXPAND|FILL)
        guitop.top().tooltip(self.xtext, "x coordinate of the mouse click")
        guitop.top().tooltip(self.ytext, "y coordinate of the mouse click")
        # End of clicked point display

        self.infoframe = GtkFrame()
        self.mainbox.add(self.infoframe, expand=1, fill=1)
        
        self.buildInfoGUI(self.modeclass)

        # Buttons at the bottom: Prev, Clear, Next
        buttonbox = GtkHBox()
        self.prev = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkArrow(at=ARROW_LEFT))
        lbox.add(GtkLabel(' Prev'))
        self.prev.add(lbox)
        self.prev.connect('clicked', self.prevQuery)
        guitop.top().tooltip(self.prev,
                             "Go back to the previous object")
        buttonbox.add(self.prev, expand=0, fill=0, padding=2)
        
        self.clear = GtkButton("Clear")
        self.clear.connect('clicked', self.clearQuery)
        guitop.top().tooltip(self.clear, "Clear the current query.")
        buttonbox.add(self.clear, expand=1, fill=1, padding=2)

        self.next = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkLabel('Next '), expand=0, fill=0)
        lbox.add(GtkArrow(at=ARROW_RIGHT), expand=0,fill=0)
        self.next.add(lbox)
        self.next.connect('clicked', self.nextQuery)
        guitop.top().tooltip(self.next, "Go on to the next object")
        buttonbox.add(self.next, expand=0, fill=0, padding=2)

        self.mainbox.add(buttonbox, expand=0, fill=0, padding=2)

        self.mainbox.show_all()
        self.sensitize()
        
        self.xposition = None
        self.yposition = None

        self.sbcallbacks = [
            switchboard.requestCallback((self.toolbox.gfxwindow(),
                                         "query skeleton"), self.updateQuery),
            switchboard.requestCallback('changed pixel group', self.grpchanged),
            switchboard.requestCallback('destroy pixel group', self.grpdestroy),
            switchboard.requestCallback('renamed pixel group', self.grprenamed)
            ]
        
    def buildInfoGUI(self, modeclass):
        debug.mainthreadTest()
        if self.modeobj and self.modeobj.__class__ is modeclass:
            return
        if self.modeobj:
            self.infoframe.remove(self.modeobj.gtk)
        # Reuse existing modeobj, if possible.
        try:
            self.modeobj = self.modeobjdict[modeclass]
        except KeyError:
            self.modeobj = self.modeobjdict[modeclass] = modeclass(self)
        self.infoframe.add(self.modeobj.gtk)

    def sensitize(self):
        debug.mainthreadTest()
        self.clear.set_sensitive(self.clearable())
        self.prev.set_sensitive(self.prev_able())
        self.next.set_sensitive(self.next_able())

    def clearable(self):
        try:
            return self.toolbox.querier.clearable()
        except AttributeError:
            return 0
        
    def prev_able(self):
        return self.toolbox.prev_able()

    def next_able(self):
        return self.toolbox.next_able()

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.toolbox.resetRecords()
        self.sensitize()

    def deactivate(self):
        toolboxGUI.GfxToolbox.deactivate(self)
        self.gfxwindow().removeMouseHandler()

    def close(self):
        if self.modeobj:
            self.modeobj.destroy()
        map(switchboard.removeCallback, self.sbcallbacks)
        toolboxGUI.GfxToolbox.close(self)

    def acceptEvent(self, eventtype):
        return eventtype == 'up'

    def up(self, x, y, shift, ctrl):
        self.modeobj.menucmd()(position=primitives.Point(x,y))

    def updateQuery(self):
        if not self.toolbox.querier:
            return
        # See if there's anything to update ...
        if self.toolbox.querier.object:
            # Change mode if needed.
            self.handleMode(self.toolbox.querier.targetname)
            querier = self.toolbox.querier
            self.modeobj.updateSomething(querier)
            self.showPosition(querier.position.x, querier.position.y)
        else:
            self.modeobj.updateNothing()
        self.sensitize()

    def changeMode(self, mode):
        self.modeclass = mode
        self.buildInfoGUI(mode)
        self.mainbox.show_all()
        try:
            self.toolbox.peeker.reset()
        except AttributeError:
            pass
        # If there's a recorded mouse-click position, an appropriate object
        # will be queried automatically when changing mode.
        if self.xposition is not None and self.yposition is not None:
            self.modeobj.menucmd()(position=primitives.Point(self.xposition,
                                                             self.yposition))

    def changeModeCB(self, button, mode):
        if self.modeclass is not mode:
            self.changeMode(mode)

    def blockAutoQuery(self):
        self.xposition = None
        self.yposition = None

    def changeModeWithObject(self, object, objtype):
        self.blockAutoQuery()
        self.modebuttondict[objtype].set_active(1)
        skeleton = self.toolbox.querier.skeleton
        self.modeobj.menucmd()(position=object.repr_position())

    def showPosition(self, x, y):
        debug.mainthreadTest()
        self.xtext.set_text("%-11.4g" % x)
        self.ytext.set_text("%-11.4g" % y)
        self.xposition = x
        self.yposition = y

    def prevQuery(self, gtkobj):
        prev = self.toolbox.records.prev().clone()
        targetname = prev.targetname
        self.handleMode(targetname)
        self.toolbox.querier = prev
        self.toolbox.querysignal()

    def clearQuery(self, gtkobj):
        self.clearing()

    def clearing(self):
        self.modeobj.updateNothing()
        self.toolbox.clearQuerier()
        self.sensitize()

    def nextQuery(self, gtkobj):
        next = self.toolbox.records.next().clone()
        targetname = next.targetname
        self.handleMode(targetname)
        self.toolbox.querier = next
        self.toolbox.querysignal()

    def handleMode(self, targetname):
        debug.mainthreadTest()
        for mode in modes:
            if mode.targetname==targetname:
                if self.modeclass is not mode:
                    self.blockAutoQuery()
                    self.modebuttondict[targetname].set_active(1)
                break

    # TODO: Figure out how to take advantage of ms_name.  
    def grpchanged(self, group, ms_name):
        if self.modeobj.targetname:
            self.updateQuery()
    # TODO: Figure it out here too.
    def grpdestroy(self, group, ms_name):
        if self.modeobj.targetname:
            self.updateQuery()          
    def grprenamed(self, group, oldname, newname):
        if self.modeobj.targetname:
            self.updateQuery()

def _makeGUI(self):
    return SkeletonToolboxGUI(self)

skeletoninfo.SkeletonInfoToolbox.makeGUI = _makeGUI
