# -*- python -*-
# $RCSfile: subproblemmenu.py,v $
# $Revision: 1.6 $
# $Author: redwing $
# $Date: 2005/06/10 13:13:50 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.engine import solverdriver
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import parameter
from oof2.common.IO import whoville
from oof2.engine import subproblem
from oof2.engine import timedrivers
from oof2.engine.IO import skeletongroupparams

OOF = mainmenu.OOF

subproblemMenu = OOF.Mesh.addItem(
    'Problem',
    cli_only=1,
    help='Tools for creating and solving sections of meshes',
    discussion="""<para>
    </para>""")

def __new_problem(menuitem, meshn, name):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        pblm = subproblem.SubProblem(name)
        meshcontext.getObject().addsubproblem(pblm)
    finally:
        meshcontext.cancel_reservation()

subproblemMenu.addItem(oofmenu.OOFMenuItem(
    'New',
    callback=__new_problem,
    threadable= oofmenu.THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('name',"problem_name")
            ],
    help="Define new subproblem"
    )
                   )
def __add_equation(menuitem, meshn, problem, equation):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.addEquation(equation) ## equation is a real equation
    finally:
        meshcontext.cancel_reservation()

def __remove_equation(menuitem, meshn, problem, equation):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.deleteEquation(equation) ## equation is a real equation
    finally:
        meshcontext.cancel_reservation()

def __activate_kinetics(menuitem, meshn, problem, equation):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.activateKinetics(equation) ## equation is a real equation
    finally:
        meshcontext.cancel_reservation()

def __deactivate_kinetics(menuitem, meshn, problem, equation):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.deactivateKinetics(equation) ## equation is a real equation
    finally:
        meshcontext.cancel_reservation()
        
eqnmenu = subproblemMenu.addItem(oofmenu.OOFMenuItem('Equation',
                                               help='Activate equations'))
eqnmenu.addItem(oofmenu.OOFMenuItem(
    'Activate',
    callback=__add_equation,
    threadable= oofmenu.THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
            meshparameters.EquationParameter('equation',
                                             tip=parameter.emptyTipString)
            ],
    help="Activate an equation in subproblem"
    ))

eqnmenu.addItem(oofmenu.OOFMenuItem(
    'Deactivate',
    callback=__remove_equation,
    threadable= oofmenu.THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
            meshparameters.EquationParameter('equation',
                                             tip=parameter.emptyTipString)
            ],
    help="Deactivate an equation in subproblem"
    ))


eqnmenu.addItem(oofmenu.OOFMenuItem(
    'ActivateKinetics',
    callback=__activate_kinetics,
    threadable=oofmenu.THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
            meshparameters.EquationParameter('equation',
                                             tip=parameter.emptyTipString)
    ]
    ))

eqnmenu.addItem(oofmenu.OOFMenuItem(
    'DeactivateKinetics',
    callback=_deactivate_kinetics,
    threadable=oofmenu.THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
            meshparameters.EquationParameter('equation',
                                             tip=parameter.emptyTipString)
    ]
    ))


def __activate_field(menuitem, mesh, problem, field):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.addField (field)## field is a real field
    finally:
        meshcontext.cancel_reservation()

def __deactivate_field(menuitem, mesh, problem, field):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.deleteField(field) ## field is a real field
    finally:
        meshcontext.cancel_reservation()
        
fieldmenu = subproblemMenu.addItem(oofmenu.OOFMenuItem(
    'Field',
    help='Define and activate Fields.'
    ))

fieldmenu.addItem(oofmenu.OOFMenuItem(
    "Activate",
    threadable=oofmenu.THREADABLE,
    callback=_activate_field,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
    ],
    help="Activate a Field.  The solver finds the values of active Fields."
    ))

fieldmenu.addItem(oofmenu.OOFMenuItem(
    'Deactivate',
    threadable=oofmenu.THREADABLE,
    callback=_deactivate_field,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
            ],
    help="Deactivate a Field.  The solver finds the values of active Fields.",
    ))

solvermenu = subproblemMenu.addItem(oofmenu.OOFMenuItem(
    'Solver',
    help="Commands for setting solvers on subproblems",
    cli_only=1))

def _setsolver(menuitem, mesh, problem, matrixdriver, timedriver):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.setMatrixDriver(matrixdriver)
        prob.setTimeDriver(timedriver)
    finally:
        meshcontext.cancel_reservation()
solvermenu.addItem(oofmenu.OOFMenuItem(
    'SetSolver',
    callback=_setsolver,
    threadable= oofmenu.THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
            parameter.RegisteredParameter('matrixdriver', solverdriver.Driver,
                                          tip=parameter.emptyTipString),
            parameter.RegisteredParameter('timedriver', timedrivers.TimeDriver)
            ],
    help="Set mesh driver and time driver of subproblem"
    ))


def _set_active_region(menuitem, mesh, problem, skeleton, group):
    meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
    meshcontext.reserve()
    try:
        prob = meshcontext.getObject().getsubproblem(problem)
        prob.setActiveRegion(skeleton, group)
    finally:
        meshcontext.cancel_reservation()


subproblemMenu.addItem(
    "SetSubMesh",
    cli_only = 1,
    callback = _set_active_region,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('problem',"problem_name"),
    whoville.WhoParameter("skeleton", whoville.getClass('Skeleton'),
                          tip=parameter.emptyTipString),
    skeletongroupparams.ElementGroupParameter("group", tip="Name of the group.")],
    help="set area (submesh) that will be solved for"
    )
