// -*- C++ -*-
// $RCSfile: elasticity.h,v $
// $Revision: 1.14 $
// $Author: redwing $
// $Date: 2004/12/17 18:03:49 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef ELASTICITY_H
#define ELASTICITY_H

#include <Python.h>
#include "engine/property.h"
#include "cijkl.h"
#include <string>

class Element;
class Flux;
class FluxData;
class Material;
class FEMesh;
class Position;
class TwoVectorField;
class SymmetricTensorFlux;
class ElementNodeIterator;
class PropertyOutput;
class OutputVal;

class Elasticity : public Property {
public:
  Elasticity(const std::string &name, PyObject *registry);
  virtual ~Elasticity() {}
  virtual void precompute();
  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator &,
			  const Flux*, FluxData*,
			  const MasterPosition&) const;
  virtual int K_integration_order(int) const;

  // elastic modulus to be defined in subclasses
  virtual const Cijkl cijkl(const FEMesh *, const Element *, 
			    const MasterPosition&) const=0;

  virtual void output(const FEMesh*, const Element*, const PropertyOutput*,
		      const MasterPosition&, OutputVal*) const;
  virtual int is_symmetric(const FEMesh*) const;
protected:
  TwoVectorField *displacement;
  SymmetricTensorFlux *stress_flux;
};

#endif
