# -*- python -*-
# $RCSfile: vigilante.py,v $
# $Revision: 1.8 $
# $Author: langer $
# $Date: 2005/05/13 18:44:08 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import debug
from oof2.common import primitives
from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO import xmlmenudump
from oof2.engine import skeletonmodifier
import random

class FixIllegal(skeletonmodifier.SkeletonModifier):
    def apply(self, oldskeleton, context):
        skel = oldskeleton.properCopy()
        suckers = skel.illegalElements()
        illegaldict = {}
        for el in suckers:
            illegaldict[el] = 1
        random.shuffle(suckers)
        nguilty = len(suckers)

        max = 222
        count = 0
        while illegaldict and count < max:
            count += 1
            self.smoothIllegalElements(skel, illegaldict, suckers)

        ndone = nguilty - len(illegaldict)
        if illegaldict:
            reporter.warn("Could not remove all illegal elements!")
        reporter.report("%d illegal element%s removed."
                        % (ndone, "s"*(ndone!=1)))

        skel.cleanUp()
        skel.checkIllegality()
        return skel

    def smoothIllegalElements(self, skel, illegaldict, suckers):
        for element in suckers:
            if illegaldict.has_key(element):
                node_indices = range(element.nnodes())
                random.shuffle(node_indices)
                for i in node_indices:
                    node = element.nodes[i]
                    if not (element.getRealAngle(i) > 0.0):  # negative angle
                        self.smoothTheNode(skel, node, illegaldict, suckers)
    
    def smoothTheNode(self, skel, node, illegaldict, suckers):
        # how many illegal elements?
        nguilty = 0
        for e in node.neighborElements():
            if illegaldict.has_key(e):
                nguilty += 1

        # move a node to its average position w.r.t. its neighbor nodes
        avgPos = primitives.Point(0.,0.)
        nbrNodes = node.neighborNodes(skel)
        for n in nbrNodes:
            avgPos += n.position()
        avgPos /= len(nbrNodes)
        skel.moveNodeTo(node, avgPos)
        
        # see if the move removed illegal elements
        still_guilty = []
        for e in node.neighborElements():
            if e.illegal():
                still_guilty.append(e)

        if nguilty <= len(still_guilty): # no improvement
            skel.moveNodeBack(node)
        else:
            # accept the move
            for e in node.neighborElements():
                estillguilty = e in still_guilty
                ewasguilty = illegaldict.has_key(e)
                if ewasguilty and not estillguilty:
                    del illegaldict[e]
                elif estillguilty and not ewasguilty:
                    suckers.append(e)
                    illegaldict[e] = 1

                
#########################

registeredclass.Registration(
    'Fix Illegal Elements', skeletonmodifier.SkeletonModifier,
    FixIllegal,
    ordering=99,
    ok_illegal = 1,  # It can deal with illegal skeletons 
    tip = 'Remove illegal elements from a Skeleton.',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/fix_illegal.xml'))
